#!/usr/bin/env python
# -*- coding: utf-8 -*-
#

# file      : build_pot_file.py
# project   : createbasepy
# copyright : (C) 2014 by Claude Bulin
#
# Internationalization of project
#
# This file is part of createbasepy project
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; version 3.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA


import os, sys, subprocess, gettext, datetime

from fconfig import FConfig
import Language_Init
from prefs import Preferences

try:
	import polib
except:
	if sys.stdout.isatty() == True:
		print( "")
		print( "bpf (Build Pot File) ")
		print( ">>>  " + _("Please install python-polib module") + "  <<<" )
		print( "")
	exit(0)

import urllib
import urllib.request
import http.client
import socket
from datetime import date


class build_pot_file():

	def __init__( self, Form=None, langage_folder_path="", ListPotFile=None, ProjectName=None, ProjectVersion=None, ProjectAuthor=None ):
		self.langage_folder_path      = langage_folder_path
		self.path_po                  = os.path.join(langage_folder_path, 'po')
		self.path_po_pot              = os.path.join(langage_folder_path, 'po/pot')
		self.ListPotFile              = ListPotFile
		self.ListParse                = [ "py", "glade" ]
		self.Form                     = Form
		self.ProjectName              = ProjectName
		self.ProjectVersion           = ProjectVersion
		self.ProjectAuthor            = ProjectAuthor

	#
	# Read files [ *.py, *.glade ] and product file all.pot
	#
	def make_xgettext( self, folder_path ):
		tab_scan = []
		counter_pot = 1

		print( "Read directories:" )

		# Verif in ListParse ext = .ext
		for LP in self.ListParse :
			LP_with_point = ""
			LP_without_point = ""
			if LP[0] == '.':
				LP_with_point = LP
				LP_without_point = LP[1:]
			else:
				LP_with_point = "." + LP
				LP_without_point = LP

			tab_scan[:] = []
			for root, dirs, files in os.walk(folder_path):
				if(files):
					for f in files:
						fileName, fileExtension = os.path.splitext( f )
						if LP_with_point == fileExtension:
							if not root in tab_scan:
								tab_scan.append( root )

			if tab_scan :
				print( " "*4, LP.upper()," : ", len(tab_scan) )
				for tab in tab_scan:
					os.chdir( tab )
					command = ""
					command = command + "xgettext --keyword=_ --sort-output -o "
					command = command + "'" + self.path_po_pot + "/ALL_" + str(counter_pot) + ".pot' *" + LP_with_point
					print( " "*8, command )
					subprocess.call(command, shell=True)
					counter_pot += 1

		print( "Use sed from" + " [ " + self.path_po_pot + " ]" )
		os.chdir( self.path_po_pot )
		command = ""
		command = command + "sed -i 's/charset=CHARSET/charset=UTF-8/g' *.pot"
		print( " "*4, command )
		subprocess.call(command, shell=True)


	#
	# Add datas files.pot to all.pot file
	# msgcat be applied to all files.pot
	#
	def make_msgcat(self):
		os.chdir( self.path_po_pot )
		command = ""
		for root, dirs, files in os.walk(self.path_po_pot):
			if files:
				for f in files:
					command = command + "./" + f + " "
		if command != "":
			command = command + "-F -o " + "'" + self.path_po + "/ALL.pot'"
			command = "msgcat " + command
			print( " "*4, command )
			subprocess.call(command, shell=True)
		else:
			print( " "*4,"No file *.pot" )


	#
	# Creating file(s).po if not exist
	#
	def create_langues(self):
		# header of POT file
		header_text = ""
		header_text = header_text + '# NAME_PROJECT POT Template File.\n'
		header_text = header_text + '# Copyright (C) %s  %s\n' % (date.today().year, self.ProjectAuthor)
		header_text = header_text + '# This file is distributed under the same license as the PACKAGE package.\n'
		header_text = header_text + '# %s <ADR_MAIL>, %s.\n' % (self.ProjectAuthor, date.today().year)
		header_text = header_text + '#\n'
		header_text = header_text + 'msgid ""\n'
		header_text = header_text + 'msgstr ""\n'
		header_text = header_text + '"Project-Id-Version: %s (version: %s)\\n"\n' % (self.ProjectName, self.ProjectVersion)
		header_text = header_text + '"Report-Msgid-Bugs-To: %s <ADR_MAIL>\\n"\n' % self.ProjectAuthor
		po_date = datetime.datetime.now()
		header_text = header_text + '"POT-Creation-Date: %s\\n"\n' % po_date
		header_text = header_text + '"PO-Revision-Date: %s\\n"\n' % po_date
		header_text = header_text + '"Last-Translator: %s <ADR_MAIL>\\n"\n' % self.ProjectAuthor
		header_text = header_text + '"Language-Team: URL\\n"\n'
		header_text = header_text + '"MIME-Version: 1.0\\n"\n'
		header_text = header_text + '"Content-Type: text/plain; charset=UTF-8\\n"\n'
		header_text = header_text + '"Content-Transfer-Encoding: 8bit\\n"\n'
		os.chdir( self.path_po )
		dirs = os.listdir( self.path_po )
		bool_create = False
		for LN in self.ListPotFile:
			if not LN in dirs:
				print( " "*4, "Create", LN )
				f = open(LN, "w")
				f.write(header_text)
				f.close()
				bool_create = True
		if bool_create == False:
			print( " "*4, "The files exist" )


	#
	# Add datas all.pot to the files.p
	#
	def make_msgmerge(self):
		os.chdir(self. path_po )
		dirs = os.listdir( self.path_po )
		for file in dirs:
			if os.path.isfile(file):
				fileName, fileExtension = os.path.splitext( file )
				if ".po" == fileExtension:
					command = "msgmerge --quiet " + file + " ALL.pot -U"
					print( " "*4, command )
					subprocess.call(command, shell=True)


	#
	# Product file(s).mo from file(s).po
	#
	def make_msgfmt(self):
		bool_error = False
		os.chdir( self.path_po )
		dirs = os.listdir( self.path_po )
		for file in dirs:
			if os.path.isfile(file):
				fileName, fileExtension = os.path.splitext( file )
				if ".po" == fileExtension:
					command = "msgfmt "+ file + " --output-file=" + fileName + ".mo"
					print( " "*4, command )
					if subprocess.call(command, shell=True) != 0:
						bool_error = True
		return bool_error

	#
	# create_pot_file
	#
	def create_pot_file(self):
		bool_error_make_msgfmt = False

		if not os.path.isdir( self.path_po_pot ):
			os.makedirs( self.path_po_pot )

		print("\nRead files [ *.py, *.glade ] and product file all.pot")
		print("-------------------------------------------------------")
		self.make_xgettext( self.langage_folder_path )

		print("\nAdd datas files.pot to all.pot file\nmsgcat be applied to all files.pot")
		print("-------------------------------------------------------")
		self.make_msgcat()

		print("\nCreating file(s).po if not exist")
		print("-------------------------------------------------------")
		self.create_langues()

		print("\nAdd datas all.pot to the files.po")
		print("-------------------------------------------------------")
		self.make_msgmerge()

		print("\nProduct file(s).mo from file(s).po")
		print("-------------------------------------------------------")
		bool_error_make_msgfmt = self.make_msgfmt()

		print("\nRemove directory po/pot/")
		print("-------------------------------------------------------")
		os.chdir( self.path_po )
		command = "rm -rf pot/"
		subprocess.call(command, shell=True)

		print("\nRemove files: po/*~")
		print("-------------------------------------------------------")
		print("\n")
		os.chdir( self.path_po )
		command = "rm -f *~"
		subprocess.call(command, shell=True)

		return bool_error_make_msgfmt


